import unittest
from unittest.mock import patch
from playcric.playcricket import pc
import pandas as pd
import json
import os


class TestPlayCricket(unittest.TestCase):
    def setUp(self):
        self.api_key = 'your_api_key'
        self.site_id = 'your_site_id'
        self.pc_instance = pc(self.api_key, self.site_id)

    @patch.object(pc, '_make_api_request')
    def test_list_registered_players(self, mock_make_api_request):
        mock_make_api_request.return_value = {
            'players': [
                {
                    'player_id': 1,
                    'player_name': 'John Doe',
                    'team_id': 1,
                    'team_name': 'Team A'
                },
                {
                    'player_id': 2,
                    'player_name': 'Alice Smith',
                    'team_id': 2,
                    'team_name': 'Team B'
                }
            ]
        }
        expected_df = pd.DataFrame({
            'player_id': [1, 2],
            'player_name': ['John Doe', 'Alice Smith'],
            'team_id': [1, 2],
            'team_name': ['Team A', 'Team B']
        })
        df = self.pc_instance.list_registered_players()
        pd.testing.assert_frame_equal(df, expected_df)

    @patch.object(pc, '_make_api_request')
    def test_get_all_matches(self, mock_make_api_request):
        mock_make_api_request.return_value = {
            'matches': [
                {
                    'match_id': 1,
                    'home_team_id': 1,
                    'home_team_name': 'Team A',
                    'away_team_id': 2,
                    'away_team_name': 'Team B',
                    'competition_id': 1,
                    'competition_type': 'League',
                    'match_date': '01/01/2022',
                    'last_updated': '01/01/2022'
                },
                {
                    'match_id': 2,
                    'home_team_id': 3,
                    'home_team_name': 'Team C',
                    'away_team_id': 4,
                    'away_team_name': 'Team D',
                    'competition_id': 2,
                    'competition_type': 'Cup',
                    'match_date': '02/01/2022',
                    'last_updated': '02/01/2022'
                }
            ]
        }
        expected_df = pd.DataFrame({
            'match_id': [1, 2],
            'home_team_id': [1, 3],
            'home_team_name': ['Team A', 'Team C'],
            'away_team_id': [2, 4],
            'away_team_name': ['Team B', 'Team D'],
            'competition_id': [1, 2],
            'competition_type': ['League', 'Cup'],
            'match_date': pd.to_datetime(['2022-01-01', '2022-01-02']),
            'last_updated': pd.to_datetime(['2022-01-01', '2022-01-02'])
        })
        df = self.pc_instance.get_all_matches(season=2022)
        pd.testing.assert_frame_equal(df, expected_df)

    @patch.object(pc, '_make_api_request')
    def test_get_league_table(self, mock_make_api_request):
        mock_make_api_request.return_value = {
            'league_table': [
                {
                    'values': [
                        {'col1': 1,
                            'col2': 'Team A',
                            'col3': 1,
                            'col4': 1,
                            'col5': 0,
                            'col6': 0,
                            'col7': 3
                         },
                        {'col1': 2,
                            'col2': 'Team B',
                            'col3': 1,
                            'col4': 0,
                            'col5': 0,
                            'col6': 1,
                            'col7': 0
                         }
                    ],
                    'headings': {
                        'col1': 'POSITION',
                        'col2': 'TEAM',
                        'col3': 'P',
                        'col4': 'W',
                        'col5': 'D',
                        'col6': 'L',
                        'col7': 'PTS'
                    },
                    'key': 'POSITION,TEAM,P,W,D,L,PTS'
                }
            ]
        }
        expected_df = pd.DataFrame({
            'POSITION': [1, 2],
            'TEAM': ['Team A', 'Team B'],
            'W': [1, 0],
            'D': [0, 0],
            'L': [0, 1],
            'PTS': [3, 0]
        })
        df, key = self.pc_instance.get_league_table(
            competition_id=1, simple=True)
        pd.testing.assert_frame_equal(df, expected_df)
        # self.assertEqual(key, ['TEAM', 'P', 'W', 'D', 'L', 'PTS'])

    @patch.object(pc, '_make_api_request')
    def test_get_match_result_string(self, mock_make_api_request):
        match_id = 1
        expected_result = 'Match result description'
        mock_make_api_request.return_value = [
            {'result_description': expected_result}]

        result = self.pc_instance.get_match_result_string(match_id)

        self.assertEqual(result, expected_result)

    @patch.object(pc, '_make_api_request')
    def test_get_result_for_my_team(self, mock_make_api_request):
        match_id = 1
        team_ids = [1, 2]
        expected_result = 'W'
        mock_make_api_request.return_value = {
            'match_details': [
                {'result_applied_to': 1,
                    'result': 'W'
                 }
            ]
        }

        result = self.pc_instance.get_result_for_my_team(match_id, team_ids)

        self.assertEqual(result, expected_result)

    @patch.object(pc, '_make_api_request')
    def test_get_innings_total_scores(self, mock_make_api_request):
        match_id = 6178722
        TESTDATA_FILENAME = os.path.join(os.path.dirname(
            __file__), 'test_files/match_details.json')
        f = open(TESTDATA_FILENAME)

        mock_make_api_request.return_value = json.load(f)
        EXPDATA_FILENAME = os.path.join(os.path.dirname(
            __file__), 'test_files/get_innings_total_scores.PKL')
        expected_df = pd.read_pickle(EXPDATA_FILENAME)
        df = self.pc_instance.get_innings_total_scores(match_id)
        pd.testing.assert_frame_equal(df, expected_df)


if __name__ == '__main__':
    unittest.main()



from playcric.utils import u
from playcric import config
import pandas as pd
import unittest
from unittest.mock import patch


class TestUtils(unittest.TestCase):
    def setUp(self):
        self.utils = u()

    def test_add_team_name_id_and_innings(self):
        df = pd.DataFrame(index=[0])
        team_name = 'Team A'
        team_id = 1
        opposition_name = 'Team B'
        opposition_id = 2
        innings_n = 1
        match_id = 123

        expected_df = pd.DataFrame({
            'team_name': ['Team A'],
            'team_id': [1],
            'opposition_name': ['Team B'],
            'opposition_id': [2],
            'innings': [1],
            'match_id': [123]
        })

        modified_df = self.utils._add_team_name_id_and_innings(
            df, team_name, team_id, opposition_name, opposition_id, innings_n, match_id)

        pd.testing.assert_frame_equal(modified_df, expected_df)

    def test_write_bowling_string(self):
        row = {
            "wickets": 3,
            "runs": 25
        }
        expected_result = "3-25"

        result = self.utils._write_bowling_string(row)

        self.assertEqual(result, expected_result)

    def test_write_batting_string(self):
        row = {
            "runs": 50,
            "balls": 30,
            "not_out": 0
        }
        expected_result = "50(30)"

        result = self.utils._write_batting_string(row)

        self.assertEqual(result, expected_result)

    def test_get_initials_surname(self):
        # Test case 1: Full name with multiple words
        name1 = "John Doe"
        expected_result1 = "J Doe"
        result1 = self.utils._get_initials_surname(name1)
        self.assertEqual(result1, expected_result1)

        # Test case 2: Full name with single word
        name2 = "Alice"
        expected_result2 = "Alice"
        result2 = self.utils._get_initials_surname(name2)
        self.assertEqual(result2, expected_result2)

        # Test case 3: Empty name
        name3 = ""
        expected_result3 = None
        result3 = self.utils._get_initials_surname(name3)
        self.assertEqual(result3, expected_result3)

        class TestUtils(unittest.TestCase):
            def setUp(self):
                self.utils = u()

    def test_standardise_bowl_with_data(self):
        # Test case with non-empty DataFrame
        bowl = pd.DataFrame({
            'bowler_name': ['John Doe', 'Alice Smith'],
            'overs': ['4.2', '3.5'],
            'runs': ['25', '15'],
            'wickets': ['2', '1'],
            'maidens': ['0', '1'],
            'no_balls': ['1', '0'],
            'wides': ['2', '1']
        })

        expected_result = pd.DataFrame({
            'bowler_name': ['John Doe', 'Alice Smith'],
            'overs': ['4.2', '3.5'],
            'runs': [25, 15],
            'wickets': [2, 1],
            'maidens': [0, 1],
            'no_balls': [1, 0],
            'wides': [2, 1],
            'initial_name': ['J Doe', 'A Smith'],
            'balls': [26, 23]
        })

        modified_bowl = self.utils._standardise_bowl(bowl)

        pd.testing.assert_frame_equal(modified_bowl, expected_result)

    def test_standardise_bowl_with_empty_data(self):
        # Test case with empty DataFrame
        bowl = pd.DataFrame()

        expected_result = pd.DataFrame(columns=config.STANDARD_BOWLING_COLS)

        modified_bowl = self.utils._standardise_bowl(bowl)

        pd.testing.assert_frame_equal(modified_bowl, expected_result)

    def test_standardise_bat_with_non_empty_data(self):
        # Test case with non-empty DataFrame
        bat = pd.DataFrame({
            'batsman_name': ['John Doe', 'Alice Smith'],
            'how_out': ['not out', 'caught'],
            'runs': ['50', '25'],
            'fours': ['5', '3'],
            'sixes': ['2', '1'],
            'balls': ['30', '20'],
            'position': ['1', '2']
        })

        expected_result = pd.DataFrame({
            'batsman_name': ['John Doe', 'Alice Smith'],
            'how_out': ['not out', 'caught'],
            'runs': [50, 25],
            'fours': [5, 3],
            'sixes': [2, 1],
            'balls': [30, 20],
            'position': [1, 2],
            'not_out': [1, 0],
            'initial_name': ['J Doe', 'A Smith']
        })

        modified_bat = self.utils._standardise_bat(bat)

        pd.testing.assert_frame_equal(modified_bat, expected_result)

    def test_standardise_bat_with_empty_data(self):
        # Test case with empty DataFrame
        bat = pd.DataFrame()

        expected_result = pd.DataFrame(columns=config.STANDARD_BATTING_COLS)

        modified_bat = self.utils._standardise_bat(bat)

        pd.testing.assert_frame_equal(modified_bat, expected_result)

    def test_get_result_letter(self):
        data = {
            'result': 'W',
            'result_applied_to': 4
        }
        team_ids = [1, 2]

        result_letter = self.utils._get_result_letter(data, team_ids)

        self.assertEqual(result_letter, 'L')

        class TestUtils(unittest.TestCase):
            def setUp(self):
                self.utils = u()

    def test_clean_league_table_simple(self):
        # Test case with simple cleaning
        df = pd.DataFrame({
            'position': [1, 2, 3],
            'team': ['Team A', 'Team B', 'Team C'],
            'tw': [5, 4, 3],
            'wd': [1, 2, 3],
            'l': [0, 1, 2],
            'pts': [10, 8, 6]
        })

        expected_result = pd.DataFrame({
            'POSITION': [1, 2, 3],
            'TEAM': ['Team A', 'Team B', 'Team C'],
            'W': [5, 4, 3],
            'D': [1, 2, 3],
            'L': [0, 1, 2],
            'PTS': [10, 8, 6]
        })

        modified_df = self.utils._clean_league_table(
            df, simple=True, key='W - Total wins')

        pd.testing.assert_frame_equal(modified_df, expected_result)

    def test_clean_league_table_advanced(self):
        # Test case with advanced cleaning
        df = pd.DataFrame({
            'position': [1, 2, 3],
            'team': ['Team A', 'Team B', 'Team C'],
            'tw': [5, 4, 3],
            'wd': [1, 2, 3],
            'l': [0, 1, 2],
            'pts': [10, 8, 6]
        })

        expected_result = pd.DataFrame({
            'POSITION': [1, 2, 3],
            'TEAM': ['Team A', 'Team B', 'Team C'],
            'TW': [5, 4, 3],
            'WD': [1, 2, 3],
            'L': [0, 1, 2],
            'PTS': [10, 8, 6]
        })

        wins = config.LEAGUE_TABLE_WIN_TYPES
        draws = config.LEAGUE_TABLE_DRAW_TYPES
        losses = config.LEAGUE_TABLE_LOSS_TYPES

        for col in wins+draws+losses:
            if col not in expected_result.columns:
                expected_result[col] = 0

        modified_df = self.utils._clean_league_table(
            df, simple=False, key='WD - Total wins and draws')

        pd.testing.assert_frame_equal(modified_df, expected_result)

    @patch('requests.get')
    def test_make_api_request_success(self, mock_get):
        # Mock the response from the API
        mock_response = {
            'status': 'success',
            'data': {
                'id': 1,
                'name': 'Team A'
            }
        }
        mock_get.return_value.status_code = 200
        mock_get.return_value.json.return_value = mock_response

        # Make the API request
        url = 'https://api.example.com/teams/1'
        response = self.utils._make_api_request(url)

        # Assert the response
        self.assertEqual(response, mock_response)

    @patch('requests.get')
    def test_make_api_request_failure(self, mock_get):
        # Mock the response from the API
        mock_get.return_value.status_code = 404

        # Make the API request
        url = 'https://api.example.com/teams/1'

        # Assert that an exception is raised
        with self.assertRaises(Exception):
            self.utils._make_api_request(url)

    def test_convert_team_ids_to_ints(self):
        team_ids = ['1', '2', '3', '4']
        expected_result = [1, 2, 3, 4]

        result = self.utils._convert_team_ids_to_ints(team_ids)

        self.assertEqual(result, expected_result)

    def test_count_balls(self):
        # Test case 1: Valid input string with overs and balls
        input_str1 = '4.2'
        expected_result1 = 26
        result1 = self.utils._count_balls(input_str1)
        self.assertEqual(result1, expected_result1)

        # Test case 2: Valid input string with only overs
        input_str2 = '5'
        expected_result2 = 30
        result2 = self.utils._count_balls(input_str2)
        self.assertEqual(result2, expected_result2)

        # Test case 3: Valid input string with only balls
        input_str3 = '.3'
        expected_result3 = 3
        result3 = self.utils._count_balls(input_str3)
        self.assertEqual(result3, expected_result3)

        # Test case 4: Empty input string
        input_str4 = ''
        expected_result4 = 0
        result4 = self.utils._count_balls(input_str4)
        self.assertEqual(result4, expected_result4)

    def test_calculate_overs(self):
        # Test case 1: Total balls is a multiple of 6
        total_balls1 = 24
        expected_result1 = '4.0'
        result1 = self.utils._calculate_overs(total_balls1)
        self.assertEqual(result1, expected_result1)

        # Test case 2: Total balls is not a multiple of 6
        total_balls2 = 25
        expected_result2 = '4.1'
        result2 = self.utils._calculate_overs(total_balls2)
        self.assertEqual(result2, expected_result2)

        # Test case 3: Total balls is 0
        total_balls3 = 0
        expected_result3 = '0.0'
        result3 = self.utils._calculate_overs(total_balls3)
        self.assertEqual(result3, expected_result3)

    def test_clean_team_name(self):
        # Test case 1: Team name with unwanted characters and words
        self.utils.team_names = ['Team A', 'Team C']

        team1 = "Team A - Test"
        expected_result1 = "Team A"
        result1 = self.utils._clean_team_name(team1)
        self.assertEqual(result1, expected_result1)

        # Test case 2: Team name without unwanted characters and words
        team2 = "Team B"
        expected_result2 = "Team B"
        result2 = self.utils._clean_team_name(team2)
        self.assertEqual(result2, expected_result2)

        # Test case 3: Team name with banned words
        team3 = "Team C - Banned Word"
        expected_result3 = "Team C"
        result3 = self.utils._clean_team_name(team3)
        self.assertEqual(result3, expected_result3)

        # Test case 4: Team name with multiple spaces
        team4 = "Team   D     "
        expected_result4 = "Team D"
        result4 = self.utils._clean_team_name(team4)
        self.assertEqual(result4, expected_result4)

    def test_calculate_batting_average_with_non_zero_innings(self):
        # Test case with non-zero innings
        row = {
            "runs": 100,
            "innings_to_count": 10
        }
        expected_result = 10.0

        result = self.utils._calculate_batting_average(row)

        self.assertEqual(result, expected_result)

    def test_calculate_batting_average_with_zero_innings(self):
        # Test case with zero innings
        row = {
            "runs": 100,
            "innings_to_count": 0
        }
        expected_result = None

        result = self.utils._calculate_batting_average(row)

        self.assertEqual(result, expected_result)

    @patch.object(u, '_make_api_request')
    def test_get_players_used_in_match(self, mock_make_api_request):
        match_id = 1
        mock_make_api_request.return_value = {
            'match_details': [
                {'home_team_id': 1,
                    'away_team_id': 100,
                    'home_club_id': 2,
                    'away_club_id': 200,
                    'players': [
                        {
                            'home_team': [
                                {
                                    'player_id': 1,
                                    'player_name': 'John Doe',
                                    # 'team_id': 1,
                                    # 'club_id': 2,
                                    'team_name': 'Team A'
                                },
                                {
                                    'player_id': 2,
                                    'player_name': 'Alice Smith',
                                    # 'team_id': 1,
                                    # 'club_id': 2,
                                    'team_name': 'Team A'
                                }
                            ]},
                        {'away_team': [
                            {
                                'player_id': 3,
                                'player_name': 'Bob Johnson',
                                # 'team_id': 100,
                                # 'club_id': 200,
                                'team_name': 'Team B'
                            },
                            {
                                'player_id': 4,
                                'player_name': 'Eve Williams',
                                # 'team_id': 100,
                                # 'club_id': 200,

                                'team_name': 'Team B'
                            }
                        ]
                        }
                    ]
                 }
            ]
        }
        expected_df = pd.DataFrame({
            'player_id': [1, 2, 3, 4],
            'player_name': ['John Doe', 'Alice Smith', 'Bob Johnson', 'Eve Williams'],


            'team_name': ['Team A', 'Team A', 'Team B', 'Team B'],
            'team_id': [1, 1, 100, 100],
            'club_id': [2, 2, 200, 200],
            'match_id': [1, 1, 1, 1]
        })
        df = self.utils._get_players_used_in_match(
            match_id, api_key='test_api_key')
        pd.testing.assert_frame_equal(df, expected_df)


if __name__ == '__main__':
    unittest.main()



import pandas as pd
import logging
import requests
import cachecontrol


from playcric import config
from playcric.utils import u


class pc(u):
    def __init__(self, api_key, site_id, club_names: list = [], team_name_to_ids_lookup: dict = {}):
        self.api_key = api_key
        self.logger = logging.getLogger('pyplaycricket.playcricket')
        self.logger.info(f'Setting site_id as {site_id}')
        self.site_id = site_id
        self.team_names = []

    def list_registered_players(self, site_id: int = None):
        """
        Retrieves a list of registered players from the specified site.

        Args:
            site_id (int, optional): The ID of the site to retrieve players from. If not provided, the default site ID will be used.

        Returns:
            pandas.DataFrame: A DataFrame containing the registered players' information.
        """
        site_id = self._set_site_id(site_id)
        data = self._make_api_request(config.PLAYERS_URL.format(
            site_id=site_id, api_key=self.api_key))

        df = pd.json_normalize(data['players'])
        return df

    def get_all_matches(self, season: int, team_ids: list = [], competition_ids: list = [], competition_types: list = [], site_id: int = None):
        """
        Retrieves all matches based on the specified filters.

        Args:
            season (int): The season for which matches should be retrieved.
            team_ids (list, optional): A list of team IDs to filter the matches. Defaults to an empty list.
            competition_ids (list, optional): A list of competition IDs to filter the matches. Defaults to an empty list.
            competition_types (list, optional): A list of competition types to filter the matches. Defaults to an empty list.
            site_id (int, optional): The site ID to retrieve matches from. Defaults to None.

        Returns:
            pandas.DataFrame: A DataFrame containing the retrieved matches data.
        """
        site_id = self._set_site_id(site_id)
        team_ids = self._convert_team_ids_to_ints(team_ids)
        data = self._make_api_request(config.MATCHES_URL.format(
            site_id=site_id, season=season, api_key=self.api_key))

        df = pd.json_normalize(data['matches'])
        if df.empty:
            return pd.DataFrame()
        for col in ['last_updated', 'match_date']:
            if col in df.columns:
                df[col] = pd.to_datetime(df[col], format='%d/%m/%Y')
        for col in ['home_team_id', 'away_team_id']:
            if col in df.columns:
                df[col] = df[col].astype('int')
        if team_ids:
            self.logger.info(f'Filtering to team_ids: {team_ids}')
            df = df.loc[(df['home_team_id'].isin(team_ids)) |
                        (df['away_team_id'].isin(team_ids))]
        if competition_ids:
            self.logger.info(
                f'Filtering to competition_ids: {competition_ids}')
            df = df.loc[(df['competition_id'].isin(competition_ids))]
        if competition_types:
            self.logger.info(
                f'Filtering to competition_types: {competition_types}')
            df = df.loc[(df['competition_type'].isin(competition_types))]
        return df

    def get_league_table(self, competition_id: int, simple: bool = False, clean_names: bool = True):
        """
        Retrieves the league table for a given competition ID.

        Args:
            competition_id (int): The ID of the competition.
            simple (bool, optional): Flag to indicate whether to return a simplified version of the league table.
                                    Defaults to False.

        Returns:
            tuple: A tuple containing the league table dataframe and the key used for column names.

        """
        data = self._make_api_request(config.LEAGUE_TABLE_URL.format(
            competition_id=competition_id, api_key=self.api_key))

        df = pd.json_normalize(data['league_table'][0]['values'])
        df.rename(columns=data['league_table'][0]['headings'], inplace=True)
        # df = df[list(set(df.columns.tolist()))]
        key = [i.replace('&nbsp;', '')
               for i in data['league_table'][0]['key'].split(',')]
        df = self._clean_league_table(df=df, simple=simple, key=key)
        if clean_names:
            df['TEAM'] = df['TEAM'].apply(
                lambda x: self._clean_team_name(team=x))

        return df, key

    def get_match_result_string(self, match_id: int):
        """
        Retrieves the match result description for a given match ID.

        Parameters:
        - match_id (int): The ID of the match.

        Returns:
        - result_text (str): The description of the match result.
        """
        data = self._make_api_request(
            config.MATCH_DETAIL_URL.format(match_id=match_id, api_key=self.api_key))

        result_text = data[0]['result_description']
        return result_text

    def get_result_for_my_team(self, match_id: int, team_ids: list = None):
        """
        Retrieves the result letter for a given match and team(s).

        Args:
            match_id (int): The ID of the match.
            team_ids (list, optional): A list of team IDs to whom the match result should apply. Defaults to None.

        Returns:
            str: The result letter indicating the outcome of the match for the specified team(s).
        """
        team_ids = self._convert_team_ids_to_ints(team_ids)
        data = self._make_api_request(
            config.MATCH_DETAIL_URL.format(match_id=match_id, api_key=self.api_key))
        data = data['match_details'][0]
        result_letter = self._get_result_letter(data=data, team_ids=team_ids)
        return result_letter

    def get_all_players_involved(self, match_ids: list, team_ids: list = []):
        """
        Retrieves all players involved in the specified matches and teams.

        Args:
            match_ids (list): A list of match IDs.
            team_ids (list, optional): A list of team IDs. If not provided, it uses the default team IDs.

        Returns:
            pandas.DataFrame: A DataFrame containing the details of all players involved.
        """
        players = []
        for match_id in match_ids:
            players.append(self._get_players_used_in_match(
                match_id=match_id, api_key=self.api_key))
        players = pd.concat(players)

        if team_ids:
            players = players.loc[players['team_id'].isin(team_ids)]

        players = players.drop_duplicates(
            subset=['player_name', 'player_id', 'match_id'])
        # players['player_id'] = players['player_id'].astype('int')
        players.reset_index(inplace=True, drop=True)
        return players

    def get_innings_total_scores(self, match_id: int):
        """
        Retrieves the total scores for each innings of a given match.

        Args:
            match_id (int): The ID of the match.

        Returns:
            pandas.DataFrame: A DataFrame containing the total scores for each innings.
        """
        data = self._make_api_request(
            config.MATCH_DETAIL_URL.format(match_id=match_id, api_key=self.api_key))
        inn = pd.json_normalize(data['match_details'][0]['innings'])
        inn = inn.drop(
            columns=['bat', 'fow', 'bowl', 'innings_number'], errors='ignore').reset_index()
        inn['match_id'] = match_id
        inn['index'] += 1
        inn.rename(columns={'index': 'innings_number'}, inplace=True)
        inn.dropna(axis=0, how='all', inplace=True)
        return inn

    def get_match_partnerships(self, match_id: int):
        """
        Retrieves the partnerships data for a given match.

        Parameters:
        - match_id (int): The ID of the match.

        Returns:
        - partnerships (DataFrame): The partnerships data for the match.
        """
        data = self._make_api_request(
            config.MATCH_DETAIL_URL.format(match_id=match_id, api_key=self.api_key))
        data = data['match_details'][0]
        all_ids = [int(data['home_team_id']), int(data['away_team_id'])]
        team_name_lookup = {int(data['home_team_id']): data['home_club_name'] + ' - ' + data['home_team_name'],
                            int(data['away_team_id']): data['away_club_name'] + ' - ' + data['away_team_name']}

        innings_n = 1
        partnerships = []
        for innings in data['innings']:
            p = pd.json_normalize(innings['fow'])
            batting_name = innings['team_batting_name']
            batting_id = int(innings['team_batting_id'])
            if batting_id == all_ids[0]:
                bowling_id = all_ids[1]
            else:
                bowling_id = all_ids[0]
            bowling_name = team_name_lookup.get(bowling_id)

            p = self._add_team_name_id_and_innings(
                p, batting_name, batting_id, bowling_name, bowling_id, innings_n, match_id)
            partnerships.append(p)
            innings_n += 1

        partnerships = pd.concat(partnerships)
        if not partnerships.empty:
            partnerships['score_added'] = partnerships['runs'].astype(
                'int') - partnerships['runs'].astype('int').shift(1).fillna(0)
        else:
            partnerships['score_added'] = None
        return partnerships

    def get_individual_stats(self, match_id: int, team_ids: list = [], stat_string: bool = False):
        data = self._make_api_request(
            config.MATCH_DETAIL_URL.format(match_id=match_id, api_key=self.api_key))
        data = data['match_details'][0]
        all_ids = [int(data['home_team_id']), int(data['away_team_id'])]
        team_name_lookup = {int(data['home_team_id']): data['home_club_name'] + ' - ' + data['home_team_name'],
                            int(data['away_team_id']): data['away_club_name'] + ' - ' + data['away_team_name']}

        all_batting = []
        all_bowling = []

        innings_n = 1
        for innings in data['innings']:
            bat = pd.json_normalize(innings['bat'])
            if (bat.empty):
                continue
            batting_name = innings['team_batting_name']
            batting_id = int(innings['team_batting_id'])

            if batting_id == all_ids[0]:
                bowling_id = all_ids[1]
            else:
                bowling_id = all_ids[0]
            bowling_name = team_name_lookup.get(bowling_id)

            bowl = pd.json_normalize(innings['bowl'])
            if bowl.empty:
                continue
            bowl = self._add_team_name_id_and_innings(
                bowl, bowling_name, bowling_id, batting_name, batting_id, innings_n, match_id)

            bat = self._add_team_name_id_and_innings(
                bat, batting_name, batting_id, bowling_name, bowling_id, innings_n, match_id)

            all_batting.append(bat)
            all_bowling.append(bowl)

            innings_n += 1

        if len(all_batting) == 0:
            return pd.DataFrame(columns=config.STANDARD_BATTING_COLS), pd.DataFrame(config.STANDARD_BOWLING_COLS)

        all_batting = self._standardise_bat(pd.concat(all_batting))
        all_bowling = self._standardise_bowl(pd.concat(all_bowling))

        if team_ids:
            all_batting = all_batting.loc[all_batting['team_id'].isin(
                team_ids)]
            all_bowling = all_bowling.loc[all_bowling['team_id'].isin(
                team_ids)]

        if stat_string:
            all_batting['stat'] = all_batting.apply(
                lambda row: self._write_batting_string(row), axis=1)
            all_bowling['stat'] = all_bowling.apply(
                lambda row: self._write_bowling_string(row), axis=1)

        # all_bowling

        return all_batting, all_bowling

    def get_stat_totals(self,  match_ids: list, team_ids: list = [], group_by_team: bool = False, for_graphics: bool = False, n_players: int = 10):
        """
        Retrieves the batting, bowling, and fielding statistics for a given set of match and team IDs.

        Args:
            match_ids (list): A list of match IDs.
            team_ids (list, optional): A list of team IDs.
            for_graphics (bool, optional): A flag indicating whether the statistics are for graphics.
            n_players (int, optional): The number of players to include in the statistics.

        Returns:
            tuple: A tuple containing the batting, bowling, and fielding statistics.

        """
        batting, bowling, fielding = self.get_individual_stats_from_all_games(
            match_ids, team_ids, stat_string=False)

        batting_groupby = ['initial_name', 'batsman_name', 'batsman_id']
        bowling_groupby = ['initial_name', 'bowler_name', 'bowler_id']
        fielding_groupby = ['fielder_name', 'fielder_id']
        if group_by_team:
            batting_groupby += ['team_id']
            bowling_groupby += ['team_id']
            fielding_groupby += ['team_id']

        batting = self._aggregate_batting_stats(batting, batting_groupby)

        bowling = self._aggregate_bowling_stats(bowling, bowling_groupby)

        fielding = self._aggregate_fielding_stats(fielding, fielding_groupby)

        if for_graphics:
            batting = batting[config.STATS_TOTALS_BATTING_COLUMNS].head(
                n_players)
            bowling = bowling[config.STATS_TOTALS_BOWLING_COLUMNS].head(
                n_players)
            fielding = fielding[config.STATS_TOTALS_FIELDING_COLUMNS].head(
                n_players)

            batting = self._extract_string_for_graphic(batting)
            bowling = self._extract_string_for_graphic(bowling)
            fielding = self._extract_string_for_graphic(fielding)
        return batting, bowling, fielding

    def get_individual_stats_from_all_games(self,  match_ids: list, team_ids: list = [], stat_string: bool = False):
        """
        Retrieves individual batting, bowling, and fielding statistics from all games.

        Args:
            match_ids (list): List of match IDs.
            team_ids (list): List of team IDs.
            stat_string (str): String representing the desired statistics.

        Returns:
            tuple: A tuple containing three pandas DataFrames - batting, bowling, and fielding.
                - batting: DataFrame containing individual batting statistics.
                - bowling: DataFrame containing individual bowling statistics.
                - fielding: DataFrame containing individual fielding statistics.

        Raises:
            ValueError: If any of the match IDs fail to retrieve statistics.

        """
        batting = []
        bowling = []
        for match_id in match_ids:
            try:
                bat, bowl = self.get_individual_stats(
                    match_id=match_id, stat_string=stat_string)
            except Exception as e:
                raise ValueError(f'MATCH ID {match_id} FAILED WITH: {e}')
            batting.append(bat)
            bowling.append(bowl)

        batting = pd.concat(batting)
        bowling = pd.concat(bowling)
        bowling['wickets'] = bowling['wickets'].fillna(0).astype('int')
        fielding = batting

        if team_ids:
            fielding = batting.loc[~batting['team_id'].isin(team_ids)]
            batting = batting.loc[batting['team_id'].isin(team_ids)]
            bowling = bowling.loc[bowling['team_id'].isin(team_ids)]

        batting.sort_values(['runs', 'balls'], ascending=[
                            False, True], inplace=True)
        bowling.sort_values(['wickets', 'runs', 'balls'], ascending=[
                            False, True, True], inplace=True)

        batting.reset_index(inplace=True, drop=True)
        fielding.reset_index(inplace=True, drop=True)
        bowling.reset_index(inplace=True, drop=True)

        return batting, bowling, fielding



import unittest
from unittest.mock import patch
from playcric.alleyn import acc
from playcric.playcricket import pc
import pandas as pd
from datetime import date


class TestAlleyn(unittest.TestCase):
    def setUp(self):
        self.api_key = 'your_api_key'
        self.site_id = 'your_site_id'
        self.acc_instance = acc(self.api_key, self.site_id)

    def test_get_innings_scores(self):
        match_ids = [1, 2, 3]
        expected_team_names = "Team A\nTeam B\nTeam A\nTeam B\nTeam A\nTeam B"
        expected_innings_scores = "100-2\n150-3\n100-2\n150-3\n100-2\n150-3"
        with patch.object(self.acc_instance, '_make_api_request') as mock_make_api_request:
            mock_make_api_request.return_value = {
                'match_details': [
                    {
                        'result': True,
                        'innings': [
                            {
                                'team_batting_name': 'Team A',
                                'team_batting_id': 1,
                                'runs': 100,
                                'wickets': '2'
                            },
                            {
                                'team_batting_name': 'Team B',
                                'team_batting_id': 2,
                                'runs': 150,
                                'wickets': '3'
                            }
                        ]
                    }
                ]
            }
            team_names, innings_scores = self.acc_instance.get_innings_scores(
                match_ids)
            self.assertEqual(team_names, expected_team_names)
            self.assertEqual(innings_scores, expected_innings_scores)

    def test_make_sure_number_of_players_is_consistent(self):
        api_key = 'your_api_key'
        site_id = 'your_site_id'
        acc_instance = acc(api_key, site_id)

        names_list = ['John', 'Alice']
        stats_list = ['100', '50']
        expected_names_list = ['John', 'Alice', ' ']
        expected_stats_list = ['100', '50', ' ']

        actual_names_list, actual_stats_list = acc_instance._make_sure_number_of_players_is_consistent(
            names_list, stats_list, players_to_include=3)

        self.assertEqual(actual_names_list, expected_names_list)
        self.assertEqual(actual_stats_list, expected_stats_list)

    def test_get_result_description_and_margin(self):
        match_ids = [1]
        team_ids = [1, 10, 3]
        expected_result = "Team A Lost by 3 wickets\n"
        with patch.object(self.acc_instance, '_make_api_request') as mock_make_api_request:
            mock_make_api_request.return_value = {
                'match_details': [
                    {
                        'result': 'W',
                        'home_team_id': 1,
                        'home_team_name': 'Team A',
                        'away_team_id': 2,
                        'away_team_name': 'Team B',
                        'innings': [
                            {
                                'team_batting_name': 'Team A',
                                'team_batting_id': 1,
                                'runs': 100,
                                'wickets': '8'
                            },
                            {
                                'team_batting_name': 'Team B',
                                'team_batting_id': 2,
                                'runs': 101,
                                'wickets': '7'
                            }
                        ],
                        'result_applied_to': 2,
                        'batted_first': 1
                    }
                ]
            }
            result_description_and_margin = self.acc_instance.get_result_description_and_margin(
                match_ids, team_ids)
            self.assertEqual(result_description_and_margin, expected_result)

    def test_add_to_stats_string(self):
        names_list = ['John', 'Alice']
        stats_list = ['100', '50']
        expected_stats_summary = 'John\nAlice\n100\n50\n'

        actual_stats_summary = self.acc_instance._add_to_stats_string(
            '', names_list, stats_list)

        self.assertEqual(actual_stats_summary, expected_stats_summary)

    def test_order_matches_for_the_graphics(self):
        matches = pd.DataFrame({
            'match_date': ['2022-01-03', '2022-01-02', '2022-01-01'],
            'home_team_id': [3, 2, 1],
            'away_team_id': [6, 5, 4],
            'home_club_name': ['Team C', 'Team B', 'Team A'],
            'away_club_name': ['Team F', 'Team E', 'Team D']
        })

        expected_order = pd.DataFrame({
            'match_date': ['2022-01-01', '2022-01-02', '2022-01-03'],
            'home_team_id': [1, 2, 3],
            'away_team_id': [4, 5, 6],
            'home_club_name': ['Team A', 'Team B', 'Team C'],
            'away_club_name': ['Team D', 'Team E', 'Team F'], 'club_team_name': [None, None, None]
        })

        ordered_matches = self.acc_instance.order_matches_for_the_graphics(
            matches)

        ordered_matches.reset_index(inplace=True, drop=True)

        self.assertEqual(ordered_matches.to_dict(), expected_order.to_dict())

    def test_get_weekend_matches(self):
        saturday = date(2022, 1, 1)
        matches = pd.DataFrame({
            'match_date': ['2022-01-01', '2022-01-02', '2022-01-03'],
            'home_team_id': [1, 2, 3],
            'away_team_id': [4, 5, 6],
            'home_club_name': ['Team A', 'Team B', 'Team C'],
            'away_club_name': ['Team D', 'Team E', 'Team F']
        })
        matches['match_date'] = pd.to_datetime(matches['match_date'])

        expected_matches = pd.DataFrame({
            'match_date': ['2022-01-01', '2022-01-02'],
            'home_team_id': [1, 2],
            'away_team_id': [4, 5],
            'home_club_name': ['Team A', 'Team B'],
            'away_club_name': ['Team D', 'Team E'], 'club_team_name': [None, None]
        })

        expected_matches['match_date'] = pd.to_datetime(
            expected_matches['match_date'])

        weekend_matches = self.acc_instance.get_weekend_matches(
            matches, saturday)

        self.assertEqual(weekend_matches.to_dict(), expected_matches.to_dict())

    def test_get_season_opposition_list(self):
        self.acc_instance.team_names = ['Team A', 'Team E', 'Team F']
        matches = pd.DataFrame({
            'match_date': ['2022-01-01', '2022-01-02', '2022-01-03'],
            'home_team_id': [1, 2, 3],
            'away_team_id': [4, 5, 6],
            'home_club_name': ['Team A', 'Team B', 'Team C'],
            'away_club_name': ['Team D', 'Team E', 'Team F']
        })

        expected_opposition_list = "Team D\nTeam B\nTeam C"

        opposition_list = self.acc_instance.get_season_opposition_list(matches)

        self.assertEqual(opposition_list, expected_opposition_list)

    def test_get_cutout_off_league_table(self):
        self.acc_instance.team_names = ['Team B']
        league_table = pd.DataFrame({
            'POSITION': ['1', '2', '3', '4', '5'],
            'TEAM': ['Team A', 'Team B', 'Team C', 'Team D', 'Team E'],
            'W': ['5', '4', '3', '2', '1'],
            'D': ['0', '1', '2', '3', '4'],
            'L': ['0', '1', '2', '3', '4'],
            'PTS': ['15', '13', '9', '7', '5']
        })

        expected_cutout = "1\nTeam A\n5\n0\n0\n15\n2\nTeam B\n4\n1\n1\n13\n3\nTeam C\n3\n2\n2\n9"

        cutout = self.acc_instance.get_cutout_off_league_table(
            league_table, n_teams=3)

        self.assertEqual(cutout, expected_cutout)

    @patch.object(pc, 'get_individual_stats_from_all_games')
    def test_get_season_stats_totals(self, mock_get_individual_stats):
        match_ids = [1, 2, 3]
        team_ids = [1, 2, 3]
        for_graphics = False
        group_by_team = False
        n_players = 10

        # Mock the return value of _get_individual_stats_from_all_games
        mock_get_individual_stats.return_value = (
            pd.DataFrame({
                'initial_name': ['John', 'Alice'],
                'batsman_name': ['John Doe', 'Alice Smith'],
                'batsman_id': [1, 2],
                'runs': [100, 50],
                'fours': [10, 5],
                'sixes': [5, 2],
                'balls': [100, 60],
                'not_out': [0, 0],
                'match_id': [1, 2],
                'position': [5, 7],
                'how_out': ['b', 'lbw']
            }),
            pd.DataFrame({
                'initial_name': ['Bob', 'Eve'],
                'bowler_name': ['Bob Smith', 'Eve Johnson'],
                'bowler_id': [1, 2],
                'wickets': [5, 3],
                'balls': [60, 50],
                'maidens': [1, 0],
                'runs': [30, 20],
                'match_id': [1, 2]
            }),
            pd.DataFrame({
                'fielder_name': ['John Doe', 'Alice Smith', 'John Doe'],
                'fielder_id': [1, 2, 1],
                'match_id': [1, 2, 3]
            })
        )

        expected_batting = pd.DataFrame({
            'rank': [1, 2],
            'initial_name': ['John', 'Alice'],
            'batsman_name': ['John Doe', 'Alice Smith'],
            'batsman_id': [1, 2],
            'runs': [100, 50],
            'top_score': [100, 50],
            '50s': [0, 1],
            '100s': [1, 0],
            'fours': [10, 5],
            'sixes': [5, 2],
            'balls': [100, 60],
            'not_out': [0, 0],
            'match_id': [1, 1],
            'position': [5.0, 7.0],
            'innings_to_count': [1, 1],
            'average': [100.0, 50.0]
        })
        expected_bowling = pd.DataFrame({
            'rank': [1, 2],
            'initial_name': ['Bob', 'Eve'],
            'bowler_name': ['Bob Smith', 'Eve Johnson'],
            'bowler_id': [1, 2],
            'wickets': [5, 3],
            'max_wickets': [5, 3],
            '5fers': [1, 0],
            'balls': [60, 50],
            'maidens': [1, 0],
            'runs': [30, 20],
            'match_id': [1, 1],
            'overs': ['10.0', '8.2'],
            'average': [6.0, 6.67],
            'sr': [12.0, 16.67],
            'econ': [3.0, 2.4]
        })

        expected_fielding = pd.DataFrame({
            'rank': [1, 2],
            'fielder_name': ['John Doe', 'Alice Smith'],
            'fielder_id': [1, 2],
            'dismissals': [2, 1],
            'n_games': [2, 1]
        })
        batting, bowling, fielding = self.acc_instance.get_alleyn_season_totals(
            match_ids=match_ids, team_ids=team_ids, group_by_team=group_by_team, for_graphics=for_graphics, n_players=n_players)

        # batting['runs'] = batting['runs'].astype('float')

        pd.testing.assert_frame_equal(batting, expected_batting)
        pd.testing.assert_frame_equal(
            bowling.round(2), expected_bowling.round(2))
        pd.testing.assert_frame_equal(fielding, expected_fielding)

    def test_extract_string_for_graphic(self):
        df = pd.DataFrame({
            'col1': [1, 2, 3],
            'col2': ['a', 'b', 'c'],
            'col3': [True, False, True]
        })

        expected_string = "1\na\nTrue\n2\nb\nFalse\n3\nc\nTrue\n"

        actual_string = self.acc_instance._extract_string_for_graphic(df)

        self.assertEqual(actual_string, expected_string)

    @patch.object(pc, 'get_individual_stats_from_all_games')
    def test_get_best_individual_performances(self, mock_get_individual_stats):
        match_ids = [1, 2, 3]
        team_ids = [1, 2, 3]
        n_players = 5
        for_graphics = False

        # Mock the return value of _get_individual_stats_from_all_games
        mock_get_individual_stats.return_value = (
            pd.DataFrame({
                'initial_name': ['John', 'Alice'],
                'batsman_name': ['John Doe', 'Alice Smith'],
                'runs': [100, 50],
                'fours': [10, 5],
                'sixes': [5, 2],
                'balls': [100, 60],
                'not_out': [0, 0],
                'match_id': [1, 2],
                'how_out': ['b', 'lbw']
            }),
            pd.DataFrame({
                'initial_name': ['Bob', 'Eve'],
                'bowler_name': ['Bob Smith', 'Eve Johnson'],
                'bowler_id': [1, 2],
                'wickets': [5, 3],
                'balls': [60, 50],
                'maidens': [1, 0],
                'runs': [30, 20],
                'match_id': [1, 2]
            }),
            pd.DataFrame({
                'fielder_name': ['John Doe', 'Alice Smith', 'John Doe'],
                'fielder_id': [1, 2, 1],
                'match_id': [1, 2, 3]
            })
        )

        expected_batting = pd.DataFrame({
            'initial_name': ['John', 'Alice'],
            'batsman_name': ['John Doe', 'Alice Smith'],
            'runs': [100, 50],
            'fours': [10, 5],
            'sixes': [5, 2],
            'balls': [100, 60],
            'not_out': [0, 0],
            'match_id': [1, 2],
            'how_out': ['b', 'lbw'],
        })

        expected_bowling = pd.DataFrame({
            'initial_name': ['Bob', 'Eve'],
            'bowler_name': ['Bob Smith', 'Eve Johnson'],
            'bowler_id': [1, 2],
            'wickets': [5, 3],
            'balls': [60, 50],
            'maidens': [1, 0],
            'runs': [30, 20],
            'match_id': [1, 2]
        })

        batting, bowling = self.acc_instance.get_best_individual_performances(
            match_ids, team_ids, n_players, for_graphics)

        pd.testing.assert_frame_equal(batting, expected_batting)
        pd.testing.assert_frame_equal(bowling, expected_bowling)

    def test_get_individual_performance_title(self):
        batting_data = {
            'initial_name': ['John', 'Alice'],
            'opposition_name': ['Team A', 'Team B']
        }
        expected_title = ['John vs Team A', 'Alice vs Team B']

        batting_df = pd.DataFrame(batting_data)

        result_df = self.acc_instance._get_individual_performance_title(
            batting_df)

        self.assertEqual(result_df['title'].tolist(), expected_title)

    @patch.object(acc, 'get_individual_stats')
    def test_get_individual_stats_from_all_games(self, mock_get_individual_stats):
        match_ids = [1, 2, 3]
        team_ids = [1, 2, 3]
        stat_string = "runs,wickets"

        # Mock the return value of get_individual_stats
        mock_get_individual_stats.side_effect = [
            (pd.DataFrame({'player_name': ['John'], 'runs': [100], 'balls': [80], 'team_id': [1]}), pd.DataFrame(
                {'player_name': ['Alice'], 'wickets': [5], 'balls': [30], 'runs': [300], 'team_id': [1]})),
            (pd.DataFrame({'player_name': ['Bob'], 'runs': [50], 'balls': [50], 'team_id': [1]}), pd.DataFrame(
                {'player_name': ['Eve'], 'wickets': [3], 'balls': [25], 'runs': [200], 'team_id': [1]})),
            (pd.DataFrame({'player_name': ['Charlie'], 'runs': [75], 'balls': [20], 'team_id': [1]}), pd.DataFrame(
                {'player_name': ['Dave'], 'wickets': [2], 'balls': [5], 'runs': [100], 'team_id': [1]}))
        ]

        expected_batting = pd.DataFrame({
            'player_name': ['John', 'Charlie', 'Bob'],
            'runs': [100, 75, 50],
            'balls': [80, 20, 50],
            'team_id': [1, 1, 1]
        })

        expected_bowling = pd.DataFrame({
            'player_name': ['Alice', 'Eve', 'Dave'],
            'wickets': [5, 3, 2],
            'balls': [30, 25, 5],
            'runs': [300, 200, 100],
            'team_id': [1, 1, 1]
        })

        batting, bowling, _ = self.acc_instance.get_individual_stats_from_all_games(
            match_ids, team_ids, stat_string)

        pd.testing.assert_frame_equal(batting, expected_batting)
        pd.testing.assert_frame_equal(bowling, expected_bowling)

    @patch.object(pc, '_get_players_used_in_match')
    def test_get_all_players_involved(self, mock_get_players_used_in_match):
        match_ids = [1, 2, 3]
        team_ids = [1, 2, 3]

        # Mock the return value of get_players_used_in_match
        mock_get_players_used_in_match.side_effect = [
            pd.DataFrame({
                'player_name': ['John Doe', 'Alice Smith'],
                'player_id': [1, 2],
                'team_id': [1, 2],
                'match_id': [1, 1]
            }),
            pd.DataFrame({
                'player_name': ['Bob Johnson', 'Eve Brown'],
                'player_id': [3, 4],
                'team_id': [1, 3],
                'match_id': [2, 2]
            }),
            pd.DataFrame({
                'player_name': ['Charlie Davis', 'Frank Wilson'],
                'player_id': [5, 6],
                'team_id': [1, 3],
                'match_id': [3, 3]
            })
        ]

        expected_players = pd.DataFrame({
            'player_name': ['John Doe', 'Alice Smith', 'Bob Johnson', 'Eve Brown', 'Charlie Davis', 'Frank Wilson'],
            'player_id': [1, 2, 3, 4, 5, 6],
            'team_id': [1, 2, 1, 3, 1, 3],
            'match_id': [1, 1, 2, 2, 3, 3]
        })

        players = self.acc_instance.get_all_players_involved(
            match_ids, team_ids)

        pd.testing.assert_frame_equal(players, expected_players)



from playcric.playcricket import pc
from playcric import config
import pandas as pd
import numpy as np
from datetime import datetime, timedelta
import logging


class acc(pc):
    def __init__(self, api_key, site_id, team_names: list = [], team_name_to_ids_lookup: dict = {}):
        """
        Initialize the Alleyn class.

        Args:
            api_key (str): The API key for accessing the Play-Cricket API.
            site_id (str): The ID of the Play-Cricket site.
            club_names (list, optional): A list of club names. Defaults to an empty list.
            team_name_to_ids_lookup (dict, optional): A dict of team name to team ID mappings. Defaults to an empty dict.
        """
        super().__init__(api_key=api_key, site_id=site_id, club_names=team_names,
                         team_name_to_ids_lookup=team_name_to_ids_lookup)
        self.logger = logging.getLogger('pyplaycricket.alleyn')
        # self.api_key = api_key
        # self.logger.info(f'Setting site_id as {site_id}')
        # self.site_id = site_id
        if not team_names:
            self.team_names = config.TEAM_NAMES
        else:
            self.team_names = team_names

        if not team_name_to_ids_lookup:
            self.team_name_to_ids_lookup = config.TEAM_NAME_TO_IDS_LOOKUP
        else:
            self.team_name_to_ids_lookup = team_name_to_ids_lookup
        self.team_ids = list(self.team_name_to_ids_lookup.values())
        self.team_ids_to_names_lookup = {
            v: k for k, v in self.team_name_to_ids_lookup.items()}

    def get_innings_scores(self, match_ids: list = []):
        """
        Retrieves the team names and innings scores for the given match IDs.

        Args:
            match_ids (list): A list of match IDs.

        Returns:
            tuple: A tuple containing the team names and innings scores as strings.
        """
        team_names = []
        innings_scores = []

        for match_id in match_ids:
            data = self._make_api_request(
                config.MATCH_DETAIL_URL.format(match_id=match_id, api_key=self.api_key))
            data = data['match_details'][0]
            if data['result']:
                for innings in data['innings']:
                    team = innings['team_batting_name']
                    team = self._clean_team_name(team)

                    team_names.append(team.strip())

                    total_runs = innings['runs']
                    if innings['wickets'] == '10':
                        wickets = ''
                    else:
                        wickets = '-' + innings['wickets']
                    score_string = f'{total_runs}{wickets}'
                    innings_scores.append(score_string)

        team_names = '\n'.join(team_names)
        innings_scores = '\n'.join(innings_scores)

        return team_names, innings_scores

    def get_individual_performances_for_graphic(self, match_ids: list = [], players_to_include: int = 3):
        """
        Retrieves individual performances for a given list of match IDs and returns a summary string.

        Args:
            match_ids (list): A list of match IDs for which individual performances are to be retrieved.
            players_to_include (int): Number of players per result to include

        Returns:
            str: A summary string containing the top batting and bowling performances for each match ID.
        """
        stats_summary = ''
        for match_id in match_ids:
            print(match_id)
            bat, bowl = self.get_individual_stats(
                match_id=match_id, stat_string=True)
            for innings in sorted(bat['innings'].unique().tolist()):
                batn = bat.loc[bat['innings'] == innings]
                batn = batn.loc[batn['how_out'] != 'did not bat']
                batn.sort_values(['runs', 'balls', 'not_out', 'position'], ascending=[
                                 False, True, False, True], inplace=True)
                batn = batn.head(players_to_include)

                batting_names = [i.upper()
                                 for i in batn['initial_name'].tolist()]
                batting_stats = batn['stat'].tolist()

                batting_names, batting_stats = self._make_sure_number_of_players_is_consistent(
                    batting_names, batting_stats, players_to_include=players_to_include)

                stats_summary = self._add_to_stats_string(
                    stats_summary, batting_names, batting_stats)

                bowln = bowl.loc[bowl['innings'] == innings]
                bowln = bowln.loc[bowln['wickets'] > 0]
                bowln.sort_values(['wickets', 'runs', 'overs'], ascending=[
                                  False, True, False], inplace=True)

                bowln = bowln.head(players_to_include)

                bowling_names = [i.upper()
                                 for i in bowln['initial_name'].tolist()]
                bowling_stats = bowln['stat'].tolist()

                bowling_names, bowling_stats = self._make_sure_number_of_players_is_consistent(
                    bowling_names, bowling_stats, players_to_include=players_to_include)

                stats_summary = self._add_to_stats_string(
                    stats_summary, bowling_names, bowling_stats)

        return stats_summary

    def _add_to_stats_string(self, stats_summary: str, names_list: list, stats_list: list):
        """
        Adds the names and stats to the given stats_summary string.

        Args:
            stats_summary (str): The current stats summary string.
            names_list (list): A list of names to be added to the stats summary.
            stats_list (list): A list of stats to be added to the stats summary.

        Returns:
            str: The updated stats summary string.
        """
        stats_summary += '\n'.join(names_list)
        stats_summary += '\n'
        stats_summary += '\n'.join(stats_list)
        stats_summary += '\n'
        return stats_summary

    def _make_sure_number_of_players_is_consistent(self, names_list: list, stats_list: list, players_to_include: int):
        """
        Ensures that the number of players is consistent by adding empty values to the lists if necessary.

        Args:
            names_list (list): A list of player names.
            stats_list (list): A list of player stats.

        Returns:
            tuple: A tuple containing the updated names_list and stats_list.
        """
        for i in range(0, players_to_include):
            if len(names_list) < players_to_include:
                names_list.append(' ')
                stats_list.append(' ')
        return names_list, stats_list

    def get_result_description_and_margin(self, match_ids: list, team_ids: list):
        """
        Retrieves the result description and margin for a list of match IDs and team IDs.

        Args:
            match_ids (list): A list of match IDs.
            team_ids (list): A list of team IDs.

        Returns:
            str: A string containing the result description and margin for each match.

        """
        all_result_strings = ''
        for match_id in match_ids:
            # print(match_id)
            self.logger.info(f'Match ID: {match_id}')
            data = self._make_api_request(
                config.MATCH_DETAIL_URL.format(match_id=match_id, api_key=self.api_key))
            data = data['match_details'][0]
            if int(data['home_team_id']) in team_ids:
                team_name = data['home_team_name']
            else:
                team_name = data['away_team_name']
            result_letter = self._get_result_letter(
                data=data, team_ids=team_ids)

            result_string = team_name + ' ' + \
                config.RESULTS_TEXT.get(result_letter)
            if result_letter == 'D':
                result_string = 'Match drawn'
            elif result_letter in config.NEUTRAL_RESULTS:
                pass
            elif data['batted_first'] == data['result_applied_to']:
                n_runs = int(data['innings'][0]['runs']) - \
                    int(data['innings'][1]['runs'])
                result_string += f' {n_runs} runs'

            else:
                n_wickets = 10 - int(data['innings'][1]['wickets'])
                result_string += f' {n_wickets} wickets'

            result_string += '\n'
            # print(result_string)
            all_result_strings += result_string
        return all_result_strings

    def order_matches_for_the_graphics(self, matches: pd.DataFrame):
        """
        Orders the matches for the graphics.

        Args:
            matches (pd.DataFrame): The DataFrame containing the matches data.

        Returns:
            pd.DataFrame: The DataFrame with matches ordered by match date and club team name.
        """
        matches['club_team_name'] = np.where(matches['home_team_id'].isin(self.team_ids), matches['home_team_id'].apply(
            lambda x: self.team_ids_to_names_lookup.get(int(x))), matches['away_team_id'].apply(lambda x: self.team_ids_to_names_lookup.get(int(x))))
        matches.sort_values(['match_date', 'club_team_name'],
                            ascending=True, inplace=True)

        return matches

    def get_weekend_matches(self, matches: pd.DataFrame, saturday: datetime):
        """
        Retrieves the matches that are scheduled for a given weekend.

        Args:
            matches (pd.DataFrame): The DataFrame containing all the matches.
            saturday (datetime): The date of the Saturday of the weekend.

        Returns:
            pd.DataFrame: The DataFrame containing the matches scheduled for the weekend.
        """
        matches = matches.loc[matches['match_date'].isin(
            [saturday, saturday+timedelta(days=1)])].copy()

        matches = self.order_matches_for_the_graphics(matches=matches)

        return matches

    def get_season_opposition_list(self, matches: pd.DataFrame):
        """
        Returns a formatted string containing the list of opposition teams for the season.

        Args:
            matches (pd.DataFrame): A DataFrame containing the match data.

        Returns:
            str: A formatted string containing the list of opposition teams.
        """
        teams_list = []
        for index, row in matches.iterrows():
            teams_list.append(row['home_club_name'])
            teams_list.append(row['away_club_name'])
        teams_list = [self._clean_team_name(i)
                      for i in teams_list if i not in self.team_names]
        teams_list = '\n'.join(teams_list)
        return teams_list

    def get_cutout_off_league_table(self, league_table: pd.DataFrame, n_teams: int = 3):
        """
        Returns a string representation of a cutout from the league table, centered around a specific team.

        Args:
            league_table (pd.DataFrame): The league table as a pandas DataFrame.
            n_teams (int, optional): The number of teams to include in the cutout. Defaults to 3.

        Returns:
            str: A string representation of the cutout from the league table.

        Raises:
            AssertionError: If n_teams is not an odd number.
            AssertionError: If there are not enough teams in the league table.
            Exception: If none of the teams in self.team_names are found in the league table.

        """
        assert (n_teams % 2 == 1, "Please choose an odd number for n_teams")
        assert (len(league_table) > n_teams, "Not enough teams in the league")

        team_index = 1000
        for team in self.team_names:
            try:
                ti = [i.split('-')[0].strip()
                      for i in league_table['TEAM'].tolist()].index(team)
            except:
                ti = 1000
            team_index = min([team_index, ti])

        if team_index == 1000:
            raise Exception(
                f"None of the teams ({','.join(self.team_names)}) in the league table")
        buffer = int((n_teams-1)/2)
        if team_index == 0:
            league_table = league_table.iloc[0:n_teams]
        else:
            league_table = league_table.iloc[max(
                [team_index - buffer, 0]):min([team_index+buffer+1, len(league_table)+1])]

        league_table['TEAM'] = league_table['TEAM'].apply(
            lambda x: self._clean_team_name(x))

        league_table_string = []
        for _, row in league_table.iterrows():
            league_table_string += [row['POSITION'], row['TEAM'],
                                    str(row['W']), str(row['D']), str(row['L']), str(row['PTS'])]
        league_table_string = '\n'.join(league_table_string)

        return league_table_string

    def get_alleyn_season_totals(self, match_ids: list, team_ids: list = [], group_by_team: bool = False, for_graphics: bool = False, n_players: int = 10):
        """
        Calculate the season statistics totals for batting, bowling, and fielding.

        Args:
            match_ids (list): List of match IDs to consider for calculating the statistics.
            team_ids (list, optional): List of team IDs to filter the statistics. Defaults to Alleyn CC adult teams.
            for_graphics (bool, optional): Flag indicating whether the statistics are for graphics. Defaults to False.
            n_players (int, optional): Number of top players to include in the statistics. Defaults to 10.

        Returns:
            tuple: A tuple containing the calculated statistics for batting, bowling, and fielding.

        """
        if not team_ids:
            team_ids = self.team_ids

        batting, bowling, fielding = self.get_stat_totals(
            match_ids=match_ids, team_ids=team_ids, group_by_team=group_by_team, for_graphics=for_graphics, n_players=n_players)
        return batting, bowling, fielding

    def _extract_string_for_graphic(self, df):
        """
        Extracts a string representation of the DataFrame for graphic display.

        Args:
            df (pandas.DataFrame): The DataFrame to extract the string representation from.

        Returns:
            str: The string representation of the DataFrame for graphic display.
        """
        string_for_graphic = ''
        for index, row in df.iterrows():
            for col in df.columns:
                string_for_graphic += str(row[col]) + '\n'
        return string_for_graphic

    def get_best_individual_performances(self, match_ids: list, team_ids: list = [], n_players=5, for_graphics: bool = False):
        """
        Retrieves the best individual performances in terms of batting and bowling for the given match and team IDs.

        Args:
            match_ids (list): List of match IDs to consider.
            team_ids (list, optional): List of team IDs to consider. If not provided, all team IDs will be considered.
            n_players (int, optional): Number of top players to retrieve. Defaults to 5.
            for_graphics (bool, optional): Flag indicating whether the results are intended for graphics. Defaults to False.

        Returns:
            tuple: A tuple containing two pandas DataFrames - `batting` and `bowling`. 
                   The `batting` DataFrame contains the best individual batting performances, 
                   while the `bowling` DataFrame contains the best individual bowling performances.
        """
        if not team_ids:
            team_ids = self.team_ids
        batting, bowling, _ = self.get_individual_stats_from_all_games(
            match_ids=match_ids, team_ids=team_ids, stat_string=True)

        if for_graphics:
            batting = self._get_individual_performance_title(
                batting)[config.INDIVIDUAL_PERFORMANCES_BATTING_COLUMNS].head(n_players)
            bowling = self._get_individual_performance_title(
                bowling)[config.INDIVIDUAL_PERFORMANCES_BOWLING_COLUMNS].head(n_players)

            batting = self._extract_string_for_graphic(batting)
            bowling = self._extract_string_for_graphic(bowling)

        return batting, bowling

    def _get_individual_performance_title(self, df):
        """
        Adds a 'title' column to the given DataFrame based on the 'initial_name' and 'opposition_name' columns.

        Args:
            df (pandas.DataFrame): The DataFrame containing the 'initial_name' and 'opposition_name' columns.

        Returns:
            pandas.DataFrame: The DataFrame with the 'title' column added.
        """
        df['title'] = df['initial_name'] + ' vs ' + df['opposition_name'].apply(
            lambda x: self._clean_team_name(x))
        return df

    def get_all_team_players_involved(self, match_ids: list, team_ids: list = []):
        """
        Retrieves all players involved in the specified matches and teams.

        Args:
            match_ids (list): A list of match IDs.
            team_ids (list, optional): A list of team IDs. If not provided, it uses the default team IDs.

        Returns:
            pandas.DataFrame: A DataFrame containing the details of all players involved.
        """
        if not team_ids:
            team_ids = self.team_ids
        players = self.get_all_players_involved(
            match_ids=match_ids, team_ids=team_ids)
        return players



from distutils.core import setup

setup(name='pyplaycricket',
      version='0.1.2',
      description='Iteracting with the playcricket API for statistical analysis and automated social media posts.',
      author='Ewan Harris',
      url='https://github.com/ewanharris12/pyplaycricket',
      long_description=open('README.md').read(),
      long_description_content_type='text/markdown'
      )
